//Copyright(c) 2003-2004, shira.
//All rights reserved.

#include <string.h>
#include <netinet/in.h>

/* IP Packet - Header - Option */
struct ip_option {         /* 32bit x N */
  unsigned char  opt_flag; /*  8bit */ /* Copied Flag + Class + Number */
  unsigned char  opt_len;  /*  8bit */ /* Option Length */
  unsigned short opt_data; /* 16bit */ /* Data */
  /*
   *  If you want more data value, please add following definition.
   *  Data length is 32bit x n.
   *  unsigned long opt_data[n];
   */
};

/* IP Packet - Header */
struct ip_header {
#if __BYTE_ORDER == __LITTLE_ENDIAN    /* Intel */
  unsigned char    ip_ihl:4;           /* header length */
  unsigned char    ip_v:4;             /* version */
#endif
#if __BYTE_ORDER == __BIG_ENDIAN       /* Motorola */
  unsigned int     ip_v:4;             /* version */
  unsigned int     ip_ihl:4;           /* header length */
#endif
  unsigned char    ip_tos;             /* type of service */
  unsigned short   ip_len;             /* total length */
  unsigned short   ip_id;              /* identification */
  unsigned short   ip_off;             /* fragment offset field */
  unsigned char    ip_ttl;             /* time to live (TTL) */
  unsigned char    ip_p;               /* protocol */
  unsigned short   ip_sum;             /* header checksum */
  struct in_addr   ip_src;             /* source ip address */
  struct in_addr   ip_dst;             /* destination ip address */
  struct ip_option ip_opt;             /* option */
};

/* IP Packet */
struct ip {
  struct ip_header iph;
  char data[65500];
};

/*
 * For Debug
 */

void protocol_id(char *, int);
void printheader(struct ip *);
void printbit(int, int);

void protocol_id(char * protocol_name, int id) {
  strcpy(protocol_name, "Unknown");
  if(id==0) strcpy(protocol_name, "IP");
  if(id==1) strcpy(protocol_name, "ICMP");
  if(id==6) strcpy(protocol_name, "TCP");
  if(id==17) strcpy(protocol_name, "UDP");
  if(id==255) strcpy(protocol_name, "RAW");
}

void printheader(struct ip * _ip) {
  char str[20];
  strcpy(str, "Unknown");
  protocol_id(str, _ip->iph.ip_p);

  printf("---------- IP Header ----------\n");
  printf("Version                : %d\n", _ip->iph.ip_v);
  printf("Internet Header Length : %d\n", _ip->iph.ip_ihl);
  printf("Total Length           : %d\n", _ip->iph.ip_tos);
  printf("Identification         : %d\n", _ip->iph.ip_len);
  printf("Fragment               : %d\n", _ip->iph.ip_off);
  printf("Time To Live           : %d\n", _ip->iph.ip_ttl);
  printf("Protocol               : %d, %s\n", _ip->iph.ip_p, str);
  printf("Checksum               : %d\n", _ip->iph.ip_sum);
  printf("Source IP              : %s\n", inet_ntoa(_ip->iph.ip_src.s_addr));
  printf("Destination IP         : %s\n", inet_ntoa(_ip->iph.ip_dst.s_addr));
  printf("---------- IP Header Option ----------\n");
  printf("Option Flag            : %d, ", _ip->iph.ip_opt.opt_flag);
  printbit(_ip->iph.ip_opt.opt_flag, 8);
  printf("\n");
  printf("Option Length          : %d\n", _ip->iph.ip_opt.opt_len);
  printf("Data                   : %d\n", _ip->iph.ip_opt.opt_data);
}

void printbit(int n, int p) {
  int i=0, z=0;

  for(i=p-1; i>=0; i--) {
    z=(n>>i)&0x01;
    printf("%1d", z);
  }
}

